@@ -5,7 +5,7 @@ Convolve is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2020, Free Software Foundation, Inc.
+Copyright (C) 2015-2021, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -520,6 +520,7 @@ twodimensionfft(struct convolveparams *p, struct fftonthreadparams *fp,
 {
   int err;
   pthread_t t;          /* All thread ids saved in this, not used. */
+  char *mmapname=NULL;
   pthread_attr_t attr;
   pthread_barrier_t b;
   size_t i, nb, *indexs, thrdcols;
@@ -539,7 +540,10 @@ twodimensionfft(struct convolveparams *p, struct fftonthreadparams *fp,
   /* ==================== */
   /* 1D FFT on each row. */
   /* ==================== */
-  gal_threads_dist_in_threads(multiple*p->ps0, nt, &indexs, &thrdcols);
+  mmapname=gal_threads_dist_in_threads(multiple*p->ps0, nt,
+                                       p->input->minmapsize,
+                                       p->cp.quietmmap,
+                                       &indexs, &thrdcols);
   if(nt==1)
     {
       fp[0].stride=1;
@@ -577,15 +581,21 @@ twodimensionfft(struct convolveparams *p, struct fftonthreadparams *fp,
       pthread_attr_destroy(&attr);
       pthread_barrier_destroy(&b);
     }
-  free(indexs);
+
+  /* Clean up. */
+  if(mmapname) gal_pointer_mmap_free(&mmapname, p->cp.quietmmap);
+  else         free(indexs);
 
 
 
   /* ====================== */
   /* 1D FFT on each column. */
   /* ====================== */
-  /* No comments, exact duplicate, except the p->ps1s! */
-  gal_threads_dist_in_threads(multiple*p->ps1, nt, &indexs, &thrdcols);
+  /* No comments, exact duplicate of above, except the p->ps1s! */
+  mmapname=gal_threads_dist_in_threads(multiple*p->ps1, nt,
+                                       p->input->minmapsize,
+                                       p->cp.quietmmap,
+                                       &indexs, &thrdcols);
   if(nt==1)
     {
       fp[0].stride=p->ps1;
@@ -614,7 +624,10 @@ twodimensionfft(struct convolveparams *p, struct fftonthreadparams *fp,
       pthread_attr_destroy(&attr);
       pthread_barrier_destroy(&b);
     }
-  free(indexs);
+
+  /* Clean up, note that 'indexs' may be memory-mapped. */
+  if(mmapname) gal_pointer_mmap_free(&mmapname, p->cp.quietmmap);
+  else         free(indexs);
 }
 
 
@@ -800,7 +813,7 @@ convolve(struct convolveparams *p)
 
   /* Save the output (which is in p->input) array. */
   if(p->input->ndim==1)
-    gal_table_write(p->input, NULL, p->cp.tableformat, p->cp.output,
+    gal_table_write(p->input, NULL, NULL, p->cp.tableformat, p->cp.output,
                     "CONVOLVED", 0);
   else
     gal_fits_img_write_to_type(p->input, cp->output, NULL, PROGRAM_NAME,
