@@ -5,7 +5,7 @@ Segment is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2020, Free Software Foundation, Inc.
+Copyright (C) 2015-2021, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -539,7 +539,7 @@ clumps_write_sn_table(struct segmentparams *p, gal_data_t *insn,
   gal_table_comments_add_intro(&comments, PROGRAM_STRING, &p->rawtime);
 
   /* write the table. */
-  gal_table_write(cols, comments, p->cp.tableformat, filename,
+  gal_table_write(cols, NULL, comments, p->cp.tableformat, filename,
                   "SKY_CLUMP_SN", 0);
 
   /* Clean up (if necessary). */
@@ -626,7 +626,8 @@ clumps_true_find_sn_thresh(struct segmentparams *p)
 
           /* Do this step. */
           gal_threads_spin_off(clumps_find_make_sn_table, &clprm,
-                               p->ltl.tottiles, p->cp.numthreads);
+                               p->ltl.tottiles, p->cp.numthreads,
+                               p->cp.minmapsize, p->cp.quietmmap);
 
           /* Set the extension name. */
           switch(clprm.step)
@@ -658,7 +659,8 @@ clumps_true_find_sn_thresh(struct segmentparams *p)
     {
       clprm.step=0;
       gal_threads_spin_off(clumps_find_make_sn_table, &clprm,
-                           p->ltl.tottiles, p->cp.numthreads);
+                           p->ltl.tottiles, p->cp.numthreads,
+                           p->cp.minmapsize, p->cp.quietmmap);
     }
 
 
