@@ -4,8 +4,9 @@
 # (in the Installing gnuastro section).
 #
 # Original author:
-#     Mohammad Akhlaghi <akhlaghi@gnu.org>
+#     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 # Contributing author(s):
+# Copyright (C) 2015-2021, Free Software Foundation, Inc.
 #
 # Copying and distribution of this file, with or without modification,
 # are permitted in any medium without royalty provided the copyright
@@ -24,7 +25,8 @@
 # file exists (basicchecks.sh is in the source tree).
 prog=arithmetic
 execname=../bin/$prog/ast$prog
-img=convolve_spatial_noised_labeled.fits
+imgin=convolve_spatial_noised.fits
+imgnc=convolve_spatial_noised_detected.fits
 
 
 
@@ -41,7 +43,8 @@ img=convolve_spatial_noised_labeled.fits
 #   - The input data was not made (for example the test that created the
 #     data file failed).
 if [ ! -f $execname ]; then echo "$execname not created."; exit 77; fi
-if [ ! -f $img      ]; then echo "$img does not exist.";   exit 77; fi
+if [ ! -f $imgin    ]; then echo "$imgin does not exist."; exit 77; fi
+if [ ! -f $imgnc    ]; then echo "$imgnc does not exist."; exit 77; fi
 
 
 
@@ -49,5 +52,9 @@ if [ ! -f $img      ]; then echo "$img does not exist.";   exit 77; fi
 
 # Actual test script
 # ==================
-$execname $img $img - $img / --hdu=1 --hdu=4 --hdu=5    \
-          --output=snimage.fits
+#
+# 'check_with_program' can be something like Valgrind or an empty
+# string. Such programs will execute the command if present and help in
+# debugging when the developer doesn't have access to the user's system.
+$check_with_program $execname $imgin $imgnc - $imgnc / --hdu=1 --hdu=SKY  \
+                               --hdu=SKY_STD  --output=snimage.fits
